package tsk::bucket;
use strict; 

sub new
{
   my ($class,$size) = @_; 
   my @list = ();
   my $self = 
   { 
      size => $size, 
      list => \@list,
      items=> 0
   }; 
   
   #if($src[$self->len -1] eq "\\")
   #{$self->len -= 1;}   
   bless $self, $class; 
   return $self; 
}

sub add
{
	my ($this, $srcFile, $destFile) = @_; 
	my $entry = "$srcFile|$destFile"; 
	push(@{$this->{'list'}},$entry); 
	$this->{'items'}++; 
}

sub items
{
	my ($this) = @_; 
	return $this->{'items'}; 
}

sub size
{
	my ($this)= @_; 
	return $this->{'items'}; 
}

sub content
{
	my ($this) = @_; 
	return @{$this->{'list'}}; 
}

sub clean
{
	my ($this) = @_;
	@{$this->('list')} = (); 
	$this->{'items'} = 0; 
}

1; 
